@@ -20,18 +20,35 @@
 #ifndef __XFCE_NOTIFY_COMMON_H__
 #define __XFCE_NOTIFY_COMMON_H__
 
-#include <glib.h>
+#include <gtk/gtk.h>
+#include <xfconf/xfconf.h>
 
 #define KNOWN_APPLICATIONS_PROP             "/applications/known_applications"
 #define MUTED_APPLICATIONS_PROP             "/applications/muted_applications"
 #define DENIED_CRITICAL_NOTIFICATIONS_PROP  "/applications/denied-critical-notifications"
 #define EXCLUDED_FROM_LOG_APPLICATIONS_PROP "/applications/excluded-from-log"
 #define MUTE_SOUNDS_PROP                    "/mute-sounds"
+#define DATETIME_FORMAT_PROP                "/date-time-format"
+#define DATETIME_CUSTOM_FORMAT_PROP         "/date-time-custom-format"
+#define LOG_MAX_SIZE_ENABLED_PROP           "/log-max-size-enabled"
+#define LOG_MAX_SIZE_PROP                   "/log-max-size"
+#define DND_ENABLED_PROP                    "/do-not-disturb"
+#define EXPIRE_TIMEOUT_ENABLED_PROP         "/expire-timeout-enabled"
+#define EXPIRE_TIMEOUT_PROP                 "/expire-timeout"
+#define EXPIRE_TIMEOUT_ALLOW_OVERRIDE_PROP  "/expire-timeout-allow-override"
+
+#define DATETIME_CUSTOM_FORMAT_DEFAULT      "%a %H:%M:%S"
+#define LOG_MAX_SIZE_DEFAULT                1000
+#define EXPIRE_TIMEOUT_DEFAULT              10
 
 G_BEGIN_DECLS
 
 gboolean xfce_notify_is_markup_valid(const gchar *markup);
 
+GtkWidget *xfce_notify_create_placeholder_label(const gchar *markup);
+
+void xfce_notify_migrate_log_max_size_setting(XfconfChannel *channel);
+
 G_END_DECLS
 
 #endif  /* __XFCE_NOTIFY_COMMON_H__ */
