@@ -36,6 +36,7 @@
 
 #include <common/xfce-notify-common.h>
 #include <common/xfce-notify-log.h>
+#include <common/xfce-notify-log-util.h>
 
 #include "notification-plugin.h"
 #include "notification-plugin-log.h"
@@ -51,7 +52,6 @@ notification_plugin_menu_item_activate (GtkWidget      *menuitem,
 
   muted = !gtk_switch_get_active (GTK_SWITCH (notification_plugin->do_not_disturb_switch));
   gtk_switch_set_active (GTK_SWITCH (notification_plugin->do_not_disturb_switch), muted);
-  notification_plugin_update_icon (notification_plugin, muted);
 }
 
 
@@ -90,15 +90,19 @@ notification_plugin_clear_log_dialog (GtkWidget *widget, gpointer user_data)
   NotificationPlugin* notification_plugin = user_data;
   GtkWidget *dialog;
 	
-	if (xfconf_channel_get_bool (notification_plugin->channel, SETTING_HIDE_CLEAR_PROMPT, FALSE))
-	{
-	  xfce_notify_log_clear ();
-	  return;
-	}
+  if (xfconf_channel_get_bool (notification_plugin->channel, SETTING_HIDE_CLEAR_PROMPT, FALSE)) {
+    xfce_notify_log_clear(notification_plugin->log);
+  } else {
+    dialog = xfce_notify_clear_log_dialog(notification_plugin->log);
+    gtk_dialog_run (GTK_DIALOG (dialog));
+    gtk_widget_destroy (dialog);
+  }
+}
 
-  dialog = xfce_notify_clear_log_dialog ();
-  gtk_dialog_run (GTK_DIALOG (dialog));
-  gtk_widget_destroy (dialog);
+
+static void
+notification_plugin_mark_all_read(GtkWidget *widget, NotificationPlugin *notification_plugin) {
+  xfce_notify_log_mark_all_read(notification_plugin->log);
 }
 
 
@@ -107,31 +111,44 @@ notification_plugin_menu_populate (NotificationPlugin *notification_plugin)
 {
   GtkMenu *menu = GTK_MENU (notification_plugin->menu);
   GtkWidget *mi, *image, *label, *box;
-  GKeyFile *notify_log;
-  gint i;
   GDateTime *today;
-  gchar *today_timestamp;
-  gsize num_groups = 0;
+  gint today_year, today_day;
   GtkCallback func = notification_plugin_menu_clear;
   gchar *notify_log_icon_folder;
   int log_icon_size;
-  gboolean state;
+  XfceDateTimeFormat dt_format;
+  gchar *custom_dt_format;
+  const gchar *show_in_menu;
+  gboolean only_unread;
+  const gchar *after_menu_shown;
+  gboolean mark_shown_read;
+  gboolean mark_all_read;
   gboolean no_notifications = FALSE;
+  GtkStyleContext *style_context = gtk_widget_get_style_context(notification_plugin->button);
   gint scale_factor = gtk_widget_get_scale_factor(notification_plugin->button);
 
-  today = g_date_time_new_now_local ();
-  today_timestamp = g_date_time_format (today, "%F");
+  today = g_date_time_new_now_local();
+  today_year = g_date_time_get_year(today);
+  today_day = g_date_time_get_day_of_year(today);
 
   /* Clean up the list and re-fill it */
   gtk_container_foreach (GTK_CONTAINER (menu), func, menu);
 
-  notify_log = xfce_notify_log_get();
   notify_log_icon_folder = xfce_resource_save_location (XFCE_RESOURCE_CACHE,
-                                                          XFCE_NOTIFY_ICON_PATH, TRUE);
+                                                        XFCE_NOTIFY_ICON_PATH, TRUE);
   log_icon_size = xfconf_channel_get_int (notification_plugin->channel,
                                           SETTING_LOG_ICON_SIZE, -1);
   if (log_icon_size == -1)
     log_icon_size = DEFAULT_LOG_ICON_SIZE;
+  dt_format = xfconf_channel_get_int(notification_plugin->channel, DATETIME_FORMAT_PROP, XFCE_DATE_TIME_FORMAT_LOCALE);
+  custom_dt_format = xfconf_channel_get_string(notification_plugin->channel, DATETIME_CUSTOM_FORMAT_PROP, DATETIME_CUSTOM_FORMAT_DEFAULT);
+
+  show_in_menu = xfconf_channel_get_string(notification_plugin->channel, SETTING_SHOW_IN_MENU, VALUE_SHOW_ALL);
+  only_unread = g_strcmp0(show_in_menu, VALUE_SHOW_UNREAD) == 0;
+
+  after_menu_shown = xfconf_channel_get_string(notification_plugin->channel, SETTING_AFTER_MENU_SHOWN, VALUE_MARK_ALL_READ);
+  mark_shown_read = g_strcmp0(after_menu_shown, VALUE_MARK_SHOWN_READ) == 0;
+  mark_all_read = g_strcmp0(after_menu_shown, VALUE_MARK_ALL_READ) == 0;
 
   /* switch for the do not disturb mode of xfce4-notifyd */
   mi = gtk_menu_item_new ();
@@ -147,10 +164,6 @@ notification_plugin_menu_populate (NotificationPlugin *notification_plugin)
                           G_OBJECT (notification_plugin->do_not_disturb_switch), "active");
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
   gtk_widget_show_all (mi);
-  /* Reset the notification status icon since all items are now read */
-  state = xfconf_channel_get_bool (notification_plugin->channel, "/do-not-disturb", FALSE);
-  notification_plugin->new_notifications = FALSE;
-  notification_plugin_update_icon (notification_plugin, state);
   g_signal_connect (mi, "activate",
                     G_CALLBACK (notification_plugin_menu_item_activate), notification_plugin);
 
@@ -159,49 +172,51 @@ notification_plugin_menu_populate (NotificationPlugin *notification_plugin)
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
   gtk_widget_show (mi);
 
-  if (notify_log) {
-    gchar **groups;
-    int log_length;
+  if (notification_plugin->log != NULL) {
+    GList *entries;
     int log_display_limit;
-    int numberof_groups;
     int numberof_notifications_shown = 0;
     gboolean log_only_today;
 
-    groups = g_key_file_get_groups (notify_log, &num_groups);
-    /* Substract 1 because the list starts with 0 */
-    numberof_groups = GPOINTER_TO_UINT(num_groups) - 1;
     log_display_limit = xfconf_channel_get_int (notification_plugin->channel,
                                                 SETTING_LOG_DISPLAY_LIMIT, -1);
     log_only_today = xfconf_channel_get_bool (notification_plugin->channel,
                                               SETTING_LOG_ONLY_TODAY, FALSE);
-
     if (log_display_limit == -1)
       log_display_limit = DEFAULT_LOG_DISPLAY_LIMIT;
-    log_length = numberof_groups - log_display_limit;
-    if (log_length < 0)
-      log_length = -1;
+
+    if (only_unread) {
+        entries = xfce_notify_log_read_unread(notification_plugin->log, NULL, log_display_limit);
+    } else {
+        entries = xfce_notify_log_read(notification_plugin->log, NULL, log_display_limit);
+    }
 
     /* Check if the menu is going to be empty despite there being a log file, e.g.
        when showing only the notifications of today but the log only contains entries
        from yesterday and before. In this case show the placeholder. */
-    if (numberof_groups == -1)
+    if (entries == NULL)
       no_notifications = TRUE;
 
-    /* Notifications are only shown until LOG_DISPLAY_LIMIT is hit */
-    for (i = numberof_groups; i > log_length; i--) {
-      const gchar *group = groups[i];
+    for (GList *l = entries; l != NULL; l = l->next) {
+      XfceNotifyLogEntry *entry = l->data;
       GtkWidget *hbox;
       GtkWidget *timestamp, *summary, *body = NULL, *app_icon = NULL;
-      gchar *app_name;
+      const gchar *app_name;
       gchar *timestamp_text;
       gchar *summary_text;
       gchar *body_text;
       cairo_surface_t *icon;
+      gchar *tooltip_timestamp_text;
       gchar *tooltip_text;
 
       /* optionally only show notifications from today */
       if (log_only_today == TRUE) {
-        if (g_ascii_strncasecmp(today_timestamp, group, 10) != 0) {
+        GDateTime *entry_local = g_date_time_to_local(entry->timestamp);
+        gint entry_year = g_date_time_get_year(entry_local);
+        gint entry_day = g_date_time_get_day_of_year(entry_local);
+        g_date_time_unref(entry_local);
+
+        if (entry_year != today_year || entry_day != today_day) {
           no_notifications = TRUE;
           continue;
         } else {
@@ -209,12 +224,13 @@ notification_plugin_menu_populate (NotificationPlugin *notification_plugin)
         }
       }
 
-      app_name = g_key_file_get_string(notify_log, group, "app_name", NULL);
-      timestamp_text = notify_log_format_timestamp(group);
-      summary_text = notify_log_format_summary(notify_log, group);
-      body_text = notify_log_format_body(notify_log, group);
-      icon = notify_log_load_icon(notify_log, group, notify_log_icon_folder, log_icon_size, scale_factor);
-      tooltip_text = notify_log_format_tooltip(app_name, timestamp_text, body_text);
+      app_name = entry->app_name != NULL ? entry->app_name : entry->app_id;
+      timestamp_text = notify_log_format_timestamp(entry->timestamp, dt_format, custom_dt_format);
+      summary_text = notify_log_format_summary(entry->summary);
+      body_text = notify_log_format_body(entry->body);
+      icon = notify_log_load_icon(notify_log_icon_folder, entry->icon_id, entry->app_id, log_icon_size, scale_factor);
+      tooltip_timestamp_text = notify_log_format_timestamp(entry->timestamp, XFCE_DATE_TIME_FORMAT_LOCALE, NULL);
+      tooltip_text = notify_log_format_tooltip(app_name, tooltip_timestamp_text, body_text);
 
       summary = g_object_new(GTK_TYPE_LABEL,
                              "use-markup", TRUE,
@@ -237,6 +253,9 @@ notification_plugin_menu_populate (NotificationPlugin *notification_plugin)
                             NULL);
       }
       if (icon != NULL) {
+        if (!entry->is_read) {
+            notify_log_icon_add_unread_emblem(icon, style_context, log_icon_size, scale_factor);
+        }
         app_icon = gtk_image_new_from_surface(icon);
       }
 
@@ -265,32 +284,46 @@ G_GNUC_END_IGNORE_DEPRECATIONS
       gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
       gtk_widget_show_all(mi);
 
-      g_free(app_name);
+      if (mark_shown_read && !entry->is_read) {
+        xfce_notify_log_mark_read(notification_plugin->log, entry->id);
+      }
+
       g_free(timestamp_text);
       g_free(summary_text);
       g_free(body_text);
+      g_free(tooltip_timestamp_text);
       g_free(tooltip_text);
       if (icon != NULL) {
           cairo_surface_destroy(icon);
       }
+      xfce_notify_log_entry_unref(entry);
     }
 
-    g_strfreev (groups);
-    g_key_file_free (notify_log);
+    g_list_free(entries);
+
     if (numberof_notifications_shown > 0)
       no_notifications = FALSE;
   }
 
-  g_free (today_timestamp);
   g_date_time_unref (today);
+  g_free(custom_dt_format);
+
+  if (notification_plugin->log != NULL && mark_all_read) {
+    xfce_notify_log_mark_all_read(notification_plugin->log);
+  }
 
   /* Show a placeholder label when there are no notifications */
-  if (!notify_log ||
-      no_notifications) {
+  if (notification_plugin->log == NULL || no_notifications) {
     GtkStyleContext *context;
     GtkBorder padding;
     mi = gtk_menu_item_new ();
-    label = gtk_label_new (_("No notifications"));
+    if (notification_plugin->log == NULL) {
+      label = gtk_label_new(_("Unable to open notification log"));
+    } else if (only_unread) {
+      label = gtk_label_new(_("No unread notifications"));
+    } else {
+      label = gtk_label_new (_("No notifications"));
+    }
     gtk_widget_set_sensitive (mi, FALSE);
     gtk_container_add (GTK_CONTAINER (mi), label);
     gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
@@ -315,10 +348,22 @@ G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), image);
 G_GNUC_END_IGNORE_DEPRECATIONS
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
+  gtk_widget_set_sensitive(mi, notification_plugin->log != NULL);
   gtk_widget_show (mi);
   g_signal_connect (mi, "activate", G_CALLBACK (notification_plugin_clear_log_dialog),
                     notification_plugin);
 
+  image = gtk_image_new_from_icon_name("notification-new-symbolic", GTK_ICON_SIZE_MENU);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+  mi = gtk_image_menu_item_new_with_mnemonic(_("_Mark all read"));
+  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), image);
+G_GNUC_END_IGNORE_DEPRECATIONS
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
+  gtk_widget_set_sensitive(mi, notification_plugin->log != NULL && !mark_all_read && notification_plugin->new_notifications);
+  gtk_widget_show(mi);
+  g_signal_connect(mi, "activate",
+                   G_CALLBACK(notification_plugin_mark_all_read), notification_plugin);
+
   mi = gtk_menu_item_new_with_mnemonic (_("_Notification settings…"));
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
   gtk_widget_show (mi);
