@@ -45,6 +45,7 @@
 
 #include <common/xfce-notify-common.h>
 #include <common/xfce-notify-log.h>
+#include <common/xfce-notify-log-util.h>
 
 #include "xfce-notify-gbus.h"
 #include "xfce-notify-daemon.h"
@@ -59,7 +60,9 @@ struct _XfceNotifyDaemon
     XfceNotifyGBusSkeleton parent;
 
     XfceNotifyOrgXfceNotifyd *xfce_iface_skeleton;
+    gboolean expire_timeout_enabled;
     gint expire_timeout;
+    gboolean expire_timeout_allow_override;
     guint bus_name_id;
     gdouble initial_opacity;
     GtkCornerType notify_location;
@@ -69,8 +72,11 @@ struct _XfceNotifyDaemon
     gboolean notification_log;
     gboolean show_text_with_gauge;
     gint primary_monitor;
+
+    XfceNotifyLog *log;
     gint log_level;
     gint log_level_apps;
+    gboolean log_max_size_enabled;
     gint log_max_size;
 
     GtkCssProvider *css_provider;
@@ -120,10 +126,22 @@ const struct {
     } default_value;
 } settings[] = {
     {
-        .name = "/expire-timeout",
+        .name = EXPIRE_TIMEOUT_ENABLED_PROP,
+        .type = G_TYPE_BOOLEAN,
+        .offset = G_STRUCT_OFFSET(XfceNotifyDaemon, expire_timeout_enabled),
+        .default_value.b = TRUE,
+    },
+    {
+        .name = EXPIRE_TIMEOUT_PROP,
         .type = G_TYPE_INT,
         .offset = G_STRUCT_OFFSET(XfceNotifyDaemon, expire_timeout),
-        .default_value.i = -1,
+        .default_value.i = EXPIRE_TIMEOUT_DEFAULT,
+    },
+    {
+        .name = EXPIRE_TIMEOUT_ALLOW_OVERRIDE_PROP,
+        .type = G_TYPE_BOOLEAN,
+        .offset = G_STRUCT_OFFSET(XfceNotifyDaemon, expire_timeout_allow_override),
+        .default_value.b = TRUE,
     },
     {
         .name = "/initial-opacity",
@@ -194,10 +212,16 @@ const struct {
         .default_value.u = 0,
     },
     {
-        .name = "/log-max-size",
+        .name = LOG_MAX_SIZE_ENABLED_PROP,
+        .type = G_TYPE_BOOLEAN,
+        .offset = G_STRUCT_OFFSET(XfceNotifyDaemon, log_max_size_enabled),
+        .default_value.b = TRUE,
+    },
+    {
+        .name = LOG_MAX_SIZE_PROP,
         .type = G_TYPE_UINT,
         .offset = G_STRUCT_OFFSET(XfceNotifyDaemon, log_max_size),
-        .default_value.u = 100,
+        .default_value.u = LOG_MAX_SIZE_DEFAULT,
     },
 };
 
@@ -308,6 +332,36 @@ xfce_notify_daemon_get_monitor_index (GdkDisplay *display,
     return 0;
 }
 
+static void
+xfce_notify_daemon_update_workarea_positions(XfceNotifyDaemon *xndaemon, gint old_nmonitor, gint new_nmonitor) {
+
+    /* Free the current reserved rectangles on screen */
+    for (gint j = 0; j < old_nmonitor; j++) {
+        g_list_free(xndaemon->reserved_rectangles[j]);
+        xndaemon->reserved_rectangles[j] = NULL;
+    }
+
+    if (old_nmonitor != new_nmonitor) {
+        // Number of monitors have changed, so realloc the arrays
+        g_free(xndaemon->reserved_rectangles);
+        xndaemon->reserved_rectangles = g_new0(GList *, new_nmonitor);
+
+        g_free(xndaemon->monitors_workarea);
+        xndaemon->monitors_workarea = g_new0(GdkRectangle, new_nmonitor);
+    }
+
+    for (gint j = 0; j < new_nmonitor; j++) {
+        GdkMonitor *monitor = gdk_display_get_monitor(gdk_display_get_default(), j);
+        DBG("Screen changed, updating workarea for monitor %d", j);
+        gdk_monitor_get_workarea(monitor, &(xndaemon->monitors_workarea[j]));
+    }
+
+    /* Traverse the active notifications tree to fill the new reserved rectangles array for screen */
+    g_tree_foreach(xndaemon->active_notifications,
+                   xfce_notify_daemon_update_reserved_rectangles,
+                   xndaemon);
+}
+
 #ifdef ENABLE_X11
 static GdkFilterReturn
 xfce_notify_rootwin_watch_workarea(GdkXEvent *gxevent,
@@ -324,14 +378,9 @@ xfce_notify_rootwin_watch_workarea(GdkXEvent *gxevent,
         GdkScreen *screen = gdk_event_get_screen(event);
         GdkDisplay *display = gdk_screen_get_display(screen);
         int nmonitor = gdk_display_get_n_monitors(display);
-        int j;
 
         DBG("got _NET_WORKAREA change on rootwin!");
-
-        for(j = 0; j < nmonitor; j++) {
-            GdkMonitor *monitor = gdk_display_get_monitor(display, j);
-            gdk_monitor_get_workarea(monitor, &(xndaemon->monitors_workarea[j]));
-        }
+        xfce_notify_daemon_update_workarea_positions(xndaemon, nmonitor, nmonitor);
     }
 
     return GDK_FILTER_CONTINUE;
@@ -344,7 +393,6 @@ xfce_notify_daemon_screen_changed(GdkScreen *screen,
 {
     XfceNotifyDaemon *xndaemon = XFCE_NOTIFY_DAEMON(user_data);
     GdkDisplay *display = gdk_screen_get_display(screen);
-    gint j;
     gint new_nmonitor;
     gint old_nmonitor;
 
@@ -360,27 +408,7 @@ xfce_notify_daemon_screen_changed(GdkScreen *screen,
     /* Set the new number of monitors */
     g_object_set_qdata(G_OBJECT(screen), XND_N_MONITORS, GINT_TO_POINTER(new_nmonitor));
 
-    /* Free the current reserved rectangles on screen */
-    for(j = 0; j < old_nmonitor; j++)
-        g_list_free(xndaemon->reserved_rectangles[j]);
-
-    g_free(xndaemon->reserved_rectangles);
-    g_free(xndaemon->monitors_workarea);
-
-    xndaemon->monitors_workarea = g_new0(GdkRectangle, new_nmonitor);
-    for(j = 0; j < new_nmonitor; j++) {
-        GdkMonitor *monitor = gdk_display_get_monitor(display, j);
-        DBG("Screen changed, updating workarea for monitor %d", j);
-        gdk_monitor_get_workarea(monitor, &(xndaemon->monitors_workarea[j]));
-    }
-
-    /* Initialize a new reserved rectangles array for screen */
-    xndaemon->reserved_rectangles = g_new0(GList *, new_nmonitor);
-
-    /* Traverse the active notifications tree to fill the new reserved rectangles array for screen */
-    g_tree_foreach(xndaemon->active_notifications,
-                   xfce_notify_daemon_update_reserved_rectangles,
-                   xndaemon);
+    xfce_notify_daemon_update_workarea_positions(xndaemon, old_nmonitor, new_nmonitor);
 }
 
 static void
@@ -497,6 +525,8 @@ static void xfce_notify_daemon_constructed (GObject *obj)
 static void
 xfce_notify_daemon_init(XfceNotifyDaemon *xndaemon)
 {
+    GError *error = NULL;
+
     xndaemon->active_notifications = g_tree_new_full(xfce_direct_compare,
                                                      NULL, NULL,
                                                      (GDestroyNotify)gtk_widget_destroy);
@@ -510,6 +540,14 @@ xfce_notify_daemon_init(XfceNotifyDaemon *xndaemon)
 
     xndaemon->denied_critical_notifications = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
     xndaemon->excluded_from_log = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
+
+    xndaemon->log = xfce_notify_log_open(&error);
+    if (xndaemon->log == NULL) {
+        g_warning("Unable to open notification log: %s", error != NULL ? error->message : "(uknknown error)");
+        if (error != NULL) {
+            g_error_free(error);
+        }
+    }
 }
 
 static void
@@ -566,6 +604,10 @@ xfce_notify_daemon_finalize(GObject *obj)
     g_hash_table_destroy(xndaemon->denied_critical_notifications);
     g_hash_table_destroy(xndaemon->excluded_from_log);
 
+    if (xndaemon->log != NULL) {
+        g_object_unref(xndaemon->log);
+    }
+
     G_OBJECT_CLASS(xfce_notify_daemon_parent_class)->finalize(obj);
 }
 
@@ -612,6 +654,14 @@ xfce_notify_daemon_window_closed(XfceNotifyWindow *window,
     list = g_list_remove(list, xfce_notify_window_get_geometry(window));
     xndaemon->reserved_rectangles[monitor] = list;
 
+    if (reason == XFCE_NOTIFY_CLOSE_REASON_DISMISSED && xndaemon->log != NULL) {
+        const gchar *log_id = xfce_notify_window_get_log_id(window);
+
+        if (log_id != NULL) {
+            xfce_notify_log_mark_read(xndaemon->log, log_id);
+        }
+    }
+
     if (G_LIKELY(id > 0)) {
         g_tree_remove(xndaemon->active_notifications, GUINT_TO_POINTER(id));
 
@@ -677,20 +727,8 @@ xfce_notify_daemon_window_size_allocate(GtkWidget *widget,
 
     if(old_geom.width != 0 && old_geom.height != 0) {
         /* Notification has already been placed previously. */
-        GdkRectangle geom_union;
-        gdk_rectangle_union(&old_geom, &geom, &geom_union);
-        if(monitor_num == old_monitor && allocation->width == old_geom.width && allocation->height == old_geom.height
-           && gdk_rectangle_equal(&geom, &geom_union) && p_screen == gtk_window_get_screen(GTK_WINDOW(widget)))
-        {
-            /* No updates are necessary */
-            return;
-        }
-        else
-        {
-            GList *old_list = xndaemon->reserved_rectangles[old_monitor];
-            old_list = g_list_remove(old_list, xfce_notify_window_get_geometry(window));
-            xndaemon->reserved_rectangles[old_monitor] = old_list;
-        }
+        xndaemon->reserved_rectangles[old_monitor] = g_list_remove(xndaemon->reserved_rectangles[old_monitor],
+                                                                   xfce_notify_window_get_geometry(window));
     }
 
     gtk_window_set_screen(GTK_WINDOW(widget), p_screen);
@@ -1063,6 +1101,52 @@ notify_application_is_muted (XfconfChannel *channel, gchar *new_app_name)
     return FALSE;
 }
 
+static gchar *
+xfce_notify_log_insert(XfceNotifyLog *log,
+                       gint log_max_size,
+                       GDateTime *timestamp,
+                       const gchar *app_id,
+                       const gchar *summary,
+                       const gchar *body,
+                       GVariant *image_data,
+                       const gchar *image_path,
+                       const gchar *app_icon,
+                       const gchar *desktop_id,
+                       gint expire_timeout,
+                       const gchar **actions)
+{
+    gchar *id = NULL;
+    XfceNotifyLogEntry *entry = xfce_notify_log_entry_new_empty();
+    entry->timestamp = g_date_time_ref(timestamp);
+    entry->app_id = g_strdup(app_id);
+    entry->icon_id = xfce_notify_log_cache_icon(image_data,
+                                                image_path,
+                                                app_icon,
+                                                desktop_id);
+    entry->summary = g_strdup(summary);
+    entry->body = g_strdup(body);
+    entry->expire_timeout = expire_timeout;
+    entry->is_read = FALSE;
+
+    for (gint i = 0; actions != NULL && actions[i] != NULL; i += 2) {
+        XfceNotifyLogEntryAction *action = g_new0(XfceNotifyLogEntryAction, 1);
+        action->id = g_strdup(actions[i]);
+        action->label = g_strdup(actions[i+1]);
+        entry->actions = g_list_prepend(entry->actions, action);
+    }
+    entry->actions = g_list_reverse(entry->actions);
+
+    xfce_notify_log_write(log, entry);
+    id = g_strdup(entry->id);
+    xfce_notify_log_entry_unref(entry);
+
+    if (log_max_size > 0) {
+        xfce_notify_log_truncate(log, log_max_size);
+    }
+
+    return id;
+}
+
 static gboolean
 notify_notify (XfceNotifyGBus *skeleton,
                GDBusMethodInvocation   *invocation,
@@ -1077,6 +1161,7 @@ notify_notify (XfceNotifyGBus *skeleton,
                XfceNotifyDaemon *xndaemon)
 {
     XfceNotifyWindow *window;
+    GDateTime *timestamp;
     GdkPixbuf *pix = NULL;
     GVariant *image_data = NULL;
     GVariant *icon_data = NULL;
@@ -1231,8 +1316,9 @@ notify_notify (XfceNotifyGBus *skeleton,
 
     notify_update_known_applications (xndaemon->settings, new_app_name);
 
-    if(expire_timeout == -1)
-        expire_timeout = xndaemon->expire_timeout;
+    if (expire_timeout == -1 || !xndaemon->expire_timeout_allow_override) {
+        expire_timeout = xndaemon->expire_timeout_enabled ? xndaemon->expire_timeout : 0;
+    }
 
     application_is_muted = notify_application_is_muted (xndaemon->settings, new_app_name);
 
@@ -1243,6 +1329,8 @@ notify_notify (XfceNotifyGBus *skeleton,
     }
 #endif
 
+    timestamp = g_date_time_new_now_local();
+
     /* Don't show notification bubbles in the "Do not disturb" mode or if the
        application has been muted by the user. Exceptions are "urgent"
        notifications. */
@@ -1259,13 +1347,24 @@ notify_notify (XfceNotifyGBus *skeleton,
                 if ((xndaemon->log_level == 0 && xndaemon->do_not_disturb == TRUE) ||
                     xndaemon->log_level == 1)
                       /* Log either all, all except muted or only muted applications */
-                      if (xndaemon->log_level_apps == 0 ||
-                          (xndaemon->log_level_apps == 1 && application_is_muted == FALSE) ||
-                          (xndaemon->log_level_apps == 2 && application_is_muted == TRUE)) {
-                          xfce_notify_log_insert (new_app_name, summary, body,
-                                                  image_data, image_path, app_icon,
-                                                  desktop_id, expire_timeout, actions,
-                                                  xndaemon->log_max_size);
+                      if (xndaemon->log != NULL
+                          && (xndaemon->log_level_apps == 0
+                              || (xndaemon->log_level_apps == 1 && application_is_muted == FALSE)
+                              || (xndaemon->log_level_apps == 2 && application_is_muted == TRUE)))
+                      {
+                          gchar *ignore_id = xfce_notify_log_insert(xndaemon->log,
+                                                                    xndaemon->log_max_size_enabled ? xndaemon->log_max_size : -1,
+                                                                    timestamp,
+                                                                    new_app_name,
+                                                                    summary,
+                                                                    body,
+                                                                    image_data,
+                                                                    image_path,
+                                                                    app_icon,
+                                                                    desktop_id,
+                                                                    expire_timeout,
+                                                                    actions);
+                          g_free(ignore_id);
                       }
             }
 
@@ -1274,6 +1373,7 @@ notify_notify (XfceNotifyGBus *skeleton,
                 g_variant_unref (image_data);
             if (desktop_id)
                 g_free (desktop_id);
+            g_date_time_unref(timestamp);
 #ifdef ENABLE_SOUND
             if (sound_props != NULL) {
                 ca_proplist_destroy(sound_props);
@@ -1355,15 +1455,26 @@ notify_notify (XfceNotifyGBus *skeleton,
     }
 
     if (xndaemon->notification_log == TRUE &&
+        xndaemon->log != NULL &&
         xndaemon->log_level == 1 &&
         xndaemon->log_level_apps <= 1 &&
         transient == FALSE &&
         !g_hash_table_contains(xndaemon->excluded_from_log, new_app_name))
     {
-        xfce_notify_log_insert (new_app_name, summary, body,
-                                image_data, image_path, app_icon,
-                                desktop_id, expire_timeout, actions,
-                                xndaemon->log_max_size);
+        gchar *log_id = xfce_notify_log_insert(xndaemon->log,
+                                               xndaemon->log_max_size_enabled ? xndaemon->log_max_size : -1,
+                                               timestamp,
+                                               new_app_name,
+                                               summary,
+                                               body,
+                                               image_data,
+                                               image_path,
+                                               app_icon,
+                                               desktop_id,
+                                               expire_timeout,
+                                               actions);
+        xfce_notify_window_set_log_id(window, log_id);
+        g_free(log_id);
     }
 
     xfce_notify_window_set_icon_only(window, x_canonical);
@@ -1391,6 +1502,8 @@ notify_notify (XfceNotifyGBus *skeleton,
       g_variant_unref (icon_data);
     if (desktop_id)
         g_free (desktop_id);
+    g_date_time_unref(timestamp);
+
     return TRUE;
 }
 
@@ -1587,6 +1700,7 @@ xfce_notify_daemon_load_config (XfceNotifyDaemon *xndaemon,
     gchar *theme;
 
     xndaemon->settings = xfconf_channel_new("xfce4-notifyd");
+    xfce_notify_migrate_log_max_size_setting(xndaemon->settings);
 
     theme = xfconf_channel_get_string(xndaemon->settings,
                                       "/theme", "Default");
