@@ -61,6 +61,7 @@ struct _XfceNotifyWindow
     GtkWindow parent;
 
     guint id;
+    gchar *log_id;
 
     GdkRectangle geometry;
     gint last_monitor;
@@ -352,14 +353,16 @@ xfce_notify_window_start_expiration(XfceNotifyWindow *window)
 static void
 xfce_notify_window_finalize(GObject *object)
 {
-#ifdef ENABLE_SOUND
     XfceNotifyWindow *window = XFCE_NOTIFY_WINDOW(object);
 
+#ifdef ENABLE_SOUND
     if (window->sound_props != NULL) {
         ca_proplist_destroy(window->sound_props);
     }
 #endif
 
+    g_free(window->log_id);
+
     G_OBJECT_CLASS(xfce_notify_window_parent_class)->finalize(object);
 }
 
@@ -377,9 +380,6 @@ xfce_notify_window_realize(GtkWidget *widget)
 
     GTK_WIDGET_CLASS(xfce_notify_window_parent_class)->realize(widget);
 
-    gdk_window_set_type_hint(gtk_widget_get_window(widget),
-                             GDK_WINDOW_TYPE_HINT_NOTIFICATION);
-    gdk_window_set_override_redirect(gtk_widget_get_window(widget), TRUE);
     xfce_notify_window_start_expiration(window);
 
 #ifdef ENABLE_SOUND
@@ -728,6 +728,23 @@ xfce_notify_window_get_id(XfceNotifyWindow *window)
     return window->id;
 }
 
+void
+xfce_notify_window_set_log_id(XfceNotifyWindow *window,
+                              const gchar *id)
+{
+    g_return_if_fail(XFCE_IS_NOTIFY_WINDOW(window));
+
+    g_free(window->log_id);
+    window->log_id = g_strdup(id);
+}
+
+const gchar *
+xfce_notify_window_get_log_id(XfceNotifyWindow *window)
+{
+    g_return_val_if_fail(XFCE_IS_NOTIFY_WINDOW(window), NULL);
+    return window->log_id;
+}
+
 void
 xfce_notify_window_set_summary(XfceNotifyWindow *window,
                                const gchar *summary)
